#include <net/SipUserAgent.h>
#include <unistd.h>
#include <signal.h>


class SipServer: public OsServerTask {
public:
	SipServer(char *ip, sipx_int_t port, sipx_int_t vb);
	~SipServer();
    virtual UtlBoolean handleMessage(OsMsg& eventMessage);
	SipUserAgent *endPoint;
private:
	sipx_int_t mvb;
	OsMsgQ* queue;

};

UtlBoolean SipServer::handleMessage(OsMsg& eventMessage)
{
    sipx_int_t msgType = eventMessage.getMsgType();
    sipx_int_t msgSubType = eventMessage.getMsgSubType();
	sipx_int_t seq;
	UtlBoolean ret = TRUE;


    if ((msgType == OsMsg::PHONE_APP) && (msgSubType == SipMessage::NET_SIP_MESSAGE)) {
		const SipMessage* sipMsg = ((SipMessageEvent&)eventMessage).getMessage();
		UtlString method;
		if (sipMsg->isResponse()) {
			return ret;
		}
		if (sipMsg->getCSeqField(&seq, &method)) {
			if (0==method.compareTo(SIP_INVITE_METHOD, UtlString::ignoreCase)) {
				if (mvb) {
					UtlString data;
					sipx_int_t len;
					sipMsg->getBytes(&data,&len);
					printf(data.data());
				}

				SipMessage Response;
				Response.setInviteBusyData(sipMsg);
				endPoint->send(Response);
			}
		}

	}

	return ret;
}


SipServer::SipServer(char *ip, sipx_int_t port, sipx_int_t vb)
{
	endPoint = new SipUserAgent(port,port,0, ip, "sipX", ip);
	queue = this->getMessageQueue();

	endPoint->addMessageObserver(*queue,
								 NULL,
								 TRUE,
								 TRUE,
								 TRUE,
								 FALSE);

	mvb = vb;
	
	endPoint->allowMethod(SIP_INVITE_METHOD);
	endPoint->allowMethod(SIP_ACK_METHOD);
	endPoint->allowMethod(SIP_CANCEL_METHOD);
	endPoint->allowMethod(SIP_BYE_METHOD);
	endPoint->allowMethod(SIP_REFER_METHOD);
	endPoint->allowMethod(SIP_OPTIONS_METHOD);
	endPoint->start();
	start();
}

SipServer::~SipServer()
{
	delete endPoint;
}


typedef void (*sighandler_t)(int);

static int R = 1;

static int handle_SIGINT(int sig)
{
	R = 0;
	return 0;
}


int main(int argc, char *argv[]) {
	SipServer *Server;
	char *ip = "127.0.0.1";
	sipx_int_t port = 5060;
	sipx_int_t vb = 0;
	time_t st = time(NULL);

	if (argc > 1) {
		ip = argv[1];
	}

	if (argc > 2) {
		port = atoi(argv[2]);
	}

	if (argc > 3) {
		vb = 1;
	}

	Server = new SipServer(ip, port, vb);
	signal(SIGINT, (sighandler_t) handle_SIGINT);

	while(R) {
		usleep(1000000);
		printf("Elapsed [%d] Tally [%d]\n", time(NULL) - st, HttpMessage::getHttpMessageCount());
		sched_yield();
	}
	printf("ShutDown\n");
	usleep(2000000);
	delete Server;
	printf("Tally [%d]\n", HttpMessage::getHttpMessageCount());
	printf("DONE\n");
}
